/* 
 * A class that exposes utility functions for the iPhone Image Capture. It is made available as a singleton and should be used
 * with the IphoneImageCapture to launch it once the RDC Config is ready.
 * 
 * Example usage: 
 * 
 *  var IphoneImageCapture = require('/mbanking/ui/checkDeposit/IphoneImageCapture');
    // Wrap imageCapture in a function so it can be launched once ImageCaptureUtils have been successfully initialized. 
    var launchImageCapture = function(){
    	IphoneImageCapture.imageCapture(imageCaptureArgs);
    };

    var ImageCaptureUtils = require('/mbanking/ui/checkDeposit/IphoneImageCaptureUtils').getInstance(launchImageCapture);
 * 
 *
 * @author Travis Crist
 */
 

// var Res = require('/shared/mbanking/modules/resources.mod');
// var Util = require('/shared/mbanking/utils/util');

// var RDC_CONFIG_FILE = "/config/rdc_configurations.json";
var _instance = null;
var _isTallDisplay = null;
var _isNormalDisplay = null;
var _deviceAspectRatio = null;
var deviceVersionObject = null;

function IphoneImageCaptureUtils() {}

// Returns the Crop Properties of the RDC Config.
IphoneImageCaptureUtils.prototype.getCropProperties = function(){
	return this.rdcConfigProperties.cropProperties;
};

// Returns the viewFinderScale Property of the RDC Config
IphoneImageCaptureUtils.prototype.getViewFinderScale = function(){
	return this.rdcConfigProperties.viewFinderScale;
};

// Returns the previewScale Property of the RDC Config
IphoneImageCaptureUtils.prototype.getPreviewScale = function(){
	return this.rdcConfigProperties.previewScale;
};

// Returns the previewRight Property of the RDC Config
IphoneImageCaptureUtils.prototype.getPreviewRight = function(){
	return this.rdcConfigProperties.previewRight;
};

// Returns the description Property of the RDC Config
IphoneImageCaptureUtils.prototype.getDescription = function(){
	return this.rdcConfigProperties.description;
};

// Returns the supportedModels Property of the RDC Config
IphoneImageCaptureUtils.prototype.getSupportedModels = function(){
	return this.rdcConfigProperties.supportedModels;
};

// Returns true if the device is an iPhone with an extended display
// Devices Currently: iPhone 5, iPhone 5S, iPhone 5C, & iPod Gen 5
IphoneImageCaptureUtils.prototype.isTallDisplay = function(){
	if(_isTallDisplay != null){
        return _isTallDisplay;
    }
    // A device with a tall display is one with a Aspect Ratio of 1.775. 
    if(getDeviceAspectRatio() == 1.775){
        _isTallDisplay = true;
    }else{
        _isTallDisplay = false;
    }
    return _isTallDisplay;
};

// Returns true if the device is an iPhone with a normal display
// Devices Currently: iPhone 3GS, 4, 4S, & iPod Gen 4.
IphoneImageCaptureUtils.prototype.isNormalDisplay = function(){
	if(_isNormalDisplay != null){
        return _isNormalDisplay;
    }
    // A device with a normal display is one with a Aspect Ratio of 1.5.
    if(getDeviceAspectRatio() == 1.5){
        _isNormalDisplay = true;
    }else{
        _isNormalDisplay = false;
    }
    return _isNormalDisplay;
};

// Calculates and returns the Device Aspect Ratio.
function getDeviceAspectRatio(){
	if(_deviceAspectRatio != null){
		return _deviceAspectRatio;
	}

	_deviceAspectRatio = (Ti.Platform.displayCaps.platformHeight/Ti.Platform.displayCaps.platformWidth);
	// Round to the nearest thousandth so that we can distinguish the iPhone 6 vs 6+ Aspect Ratios
	_deviceAspectRatio = Math.round(_deviceAspectRatio*1000)/1000;
	return _deviceAspectRatio;
}

/*
 * @param {Object} image
 *        The image blob to be cropped.
 * @param  {Object} cropProperties
 *         Properties to be used for cropping of the image. 
 *
 * returns {Object} 
 *     The cropped image Blob of the image Blob that was passed in. 
 */
IphoneImageCaptureUtils.prototype.cropImage = function(image, cropProperties){
    // Height: Height to crop image to, take 1 - the (cropRightBy+cropLeftBy) values which should be equal to the 
    //         percentage to be removed from the left & right sides of the image to preserve the aspect ratio. 
    // Width: Sets the width of the image to be cropped to so we subtract the left and
    //        the right after multiplying the width * percentage to be cropped
    //        from the images width for both sides.
    // X: Sets where to start the width calculation set it to the images width * the cropLeftBy property to crop the left side of the image. 
    // 	  The width will be used to determine how much of the right side is cropped. 
    image = image.imageAsCropped({
        height: image.height*(1 - (cropProperties.cropRightBy+cropProperties.cropLeftBy)), 
        width: image.width - (image.width*cropProperties.cropRightBy) - (image.width*cropProperties.cropLeftBy),
        x: image.width*cropProperties.cropLeftBy
    });
    return image;
};

IphoneImageCaptureUtils.prototype._initialize = function(successFn) { 

	var rdcConfigObject = null;
    var thisObj = this;

    // Res.resource(
    // 	RDC_CONFIG_FILE, 
    // 	function(responseData) {
	   //      try {
	   //      	Util.error("Successfully Loaded RDC Configurations: " + RDC_CONFIG_FILE);
	   //          rdcConfigObject = JSON.parse(responseData);
	   //      } catch(e) {
	   //          Util.error("Error parsing JSON RDC Configurations: " + RDC_CONFIG_FILE + " Using RDC Fallback Config");
	   //          rdcConfigObject = getFallbackConfig();
	   //      }
    //     	thisObj.parseRdcConfiguration(rdcConfigObject, successFn);
    // 	}, 
	   //  function(status) {
	   //      Util.error('Error getting RDC Configurations resource ' + RDC_CONFIG_FILE + " Using RDC Fallback Config");
	   //      Res.clearResource(RDC_CONFIG_FILE); 
	        rdcConfigObject = getFallbackConfig();
	        thisObj.parseRdcConfiguration(rdcConfigObject, successFn);
	   //  },
	   //  null, 
	   //  true, 
	   //  false 
    // );  
};

IphoneImageCaptureUtils.prototype.getOSVersion = function() {
	
	if (deviceVersionObject != null) {
		return deviceVersionObject;
	}
	
	var OSVersion = Ti.Platform.version;
	var versionArray = OSVersion.split(".");
	var arrayLength = versionArray.length;
	
	
	var full = (OSVersion != null) ? OSVersion : "Unknown";
	var major = (arrayLength >= 1) ? parseInt(versionArray[0], 10) : 0;
	var minor = (arrayLength >= 2) ? parseInt(versionArray[1], 10) : 0;
	var build = (arrayLength >= 3) ? parseInt(versionArray[2], 10) : 0;
	var revision = (arrayLength >= 4) ? parseInt(versionArray[3], 10) : 0;
	
	if (isNaN(major)) {
		major = 0;
	}
	
	if (isNaN(minor)) {
		minor = 0;
	}
	
	if (isNaN(build)) {
		build = 0;
	}
	
	if (isNaN(revision)) {
		revision = 0;
	}
	
	deviceVersionObject = {
		full 	 : full,
		major 	 : major,
		minor 	 : minor,
		build 	 : build,
		revision : revision
	};

	Ti.API.error("deviceVersionObject: " + JSON.stringify(deviceVersionObject));

	return deviceVersionObject;
};

/**
 * Parses the RDC Configuration passed in and assigns the properties to the this.rdcConfigProperties object for later access.
 */
IphoneImageCaptureUtils.prototype.parseRdcConfiguration = function(rdcConfig, successFn){
	try {
    	this.rdcConfigProperties = rdcConfig.iOS[this.getOSVersion().major].aspectRatio[getDeviceAspectRatio()];
    	// If the property has not been defined use the default config. 
    	if(this.rdcConfigProperties == undefined){
    		this.rdcConfigProperties = rdcConfig.DEFAULT;	
    	}
	} catch(e) {
		this.rdcConfigProperties = rdcConfig.DEFAULT;
	} 
	Ti.API.error("Aspect Ratio: " + getDeviceAspectRatio());
	Ti.API.error("rdcConfigProperties: " + JSON.stringify(this.rdcConfigProperties));
	if(typeof successFn == "function"){
		successFn();
	}
};

/**
 * Exposes a static singleton, successFn is called once the RDC Config is ready. 
 * Expects the IphoneImageCapture.imageCapture function to be passed in so that RDC Image Capture
 * Can be launched. 
 */
exports.getInstance = function(successFn) {
    if ( _instance == null ) {
        _instance = new IphoneImageCaptureUtils();
        _instance._initialize(successFn);
    }else{
    	// If we already have the config loaded and a successFn was passed in
    	// call it to launch Image Capture
    	if(typeof successFn == "function"){
			successFn();
		}
    }
    return _instance;
};

/**
 * Provides hard coded local fallback RDC Configuration in case for some reason the rdc_configurations.json resource
 * cannot be found or successfully pulled from the server.
 */
function getFallbackConfig(){
	return {
	    iOS : { 
	        6 : {
	            aspectRatio : { 
	                1.5 : {
	                    description : "iOS6_Normal_Display",
	                    supportedModels : "iPhone 3GS, iPhone 4, iPhone 4S, iPod Gen 4",
	                    cropProperties: {
	                        cropLeftBy: 0,
	                        cropRightBy: 0.0125
	                    },
	                    viewFinderScale: 1.0,
	                    previewScale: 1.01, //1.01
	                    previewRight: -5
	                }, 
	                1.775 : {
	                    description : "iOS6_Tall_Display",
	                    supportedModels : "iPhone 5, iPhone 5S, iPod Gen 5",
	                    cropProperties: {
	                        cropLeftBy: 0.065, 
	                        cropRightBy: 0
	                    },
	                    viewFinderScale: 1.15,
	                    previewScale: 1.275,
	                    previewRight: 36//58
	                }
	            }
	        },
	        7 : {
	            aspectRatio : { 
	                1.5 : {
	                    description : "iOS7_Normal_Display",
	                    supportedModels : "iPhone 4, iPhone 4S, iPod Gen 4",
	                    cropProperties : {
	                        cropLeftBy: 0, 
	                        cropRightBy: 0.125
	                    },
	                    viewFinderScale: 1.0,
	                    previewScale: 1.122,
	                    previewRight: -60
	                }, 
	                1.775 : {
	                    description : "iOS7_Tall_Display",
	                    supportedModels : "iPhone 5, iPhone 5S, iPhone 5C, iPod Gen 5",
	                    cropProperties: {
	                        cropLeftBy: 0.135, 
	                        cropRightBy: 0
	                    },
	                    viewFinderScale: 1.385, 
	                    previewScale: 1.387,
	                    previewRight: 82
	                }
	            }
	        },
	        8 : {
	            aspectRatio : { 
	                1.5 : {
	                    description : "iOS8_Normal_Display",
	                    supportedModels : "iPhone 4S",
	                    cropProperties : {
	                        cropLeftBy: 0, 
	                        cropRightBy: 0.125
	                    },
	                    viewFinderScale: 1.0,
	                    previewScale: 1.122,
	                    previewRight: -60
	                }, 
	                1.775 : {
	                    description : "iOS8_Tall_Display",
	                    supportedModels : "iPhone 5, iPhone 5S, iPhone 5C, iPod Gen 5",
	                    cropProperties: {
	                        cropLeftBy: 0.135,//0.135, 
	                        cropRightBy: 0
	                    },
	                    viewFinderScale: 1.385,//1.385, 
	                    previewScale: 1.387,//1.387,
	                    previewRight: 82//82
	                }
	            }
	        }
	    },
	    DEFAULT : {
	        description : "Default",
	        supportedModels : "Fallback for an Unknown Models",
	        cropProperties:{
	            cropLeftBy: 0, 
	            cropRightBy: 0 
	        },
	        viewFinderScale: 1.4,
	        previewScale: 1,
	        previewRight: null
	    }   
	};
};